-- http://docs.python.org/2/library/stdtypes.html#string-formatting
--  The conversion flag characters are:

--  Flag        Meaning
--  '#' The value conversion will use the “alternate form” (where defined below).
--  '0' The conversion will be zero padded for numeric values.
--  '-' The converted value is left adjusted (overrides the '0' conversion if both are given).
--  ' ' (a space) A blank should be left before a positive number (or empty string) produced by a signed conversion.
--  '+' A sign character ('+' or '-') will precede the conversion (overrides a “space” flag).
--  A length modifier (h, l, or L) may be present, but is ignored as it is not necessary for Python –
--    so e.g. %ld is identical to %d.
--
--  The conversion types are:
--
--  Conversion  Meaning Notes
--  'd' Signed integer decimal.
--  'i' Signed integer decimal.
--  'o' Signed octal value.     (1)
--  'u' Obsolete type – it is identical to 'd'. (7)
--  'x' Signed hexadecimal (lowercase). (2)
--  'X' Signed hexadecimal (uppercase). (2)
--  'e' Floating point exponential format (lowercase).  (3)
--  'E' Floating point exponential format (uppercase).  (3)
--  'f' Floating point decimal format.  (3)
--  'F' Floating point decimal format.  (3)
--  'g' Floating point format. Uses lowercase exponential format if exponent
--        is less than -4 or not less than precision, decimal format otherwise. (4)
--  'G' Floating point format. Uses uppercase exponential format if exponent
--        is less than -4 or not less than precision, decimal format otherwise. (4)
--  'c' Single character (accepts integer or single character string).
--  'r' String (converts any Python object using repr()).       (5)
--  's' String (converts any Python object using str()).        (6)
--  '%' No argument is converted, results in a '%' character in the result.

with Formatted_Output; use Formatted_Output;
procedure EssaiPrintf is
   type Enum is (Red, Black, Green);
   function "&" is new ConcatEnum (Enum);
begin
   Printf (Fmt ("Signed integer decimal %4d.\n") & 13);
   Printf (Fmt ("Signed integer decimal %4i.\n") & 34);
   Printf (Fmt ("Signed octal value %4o.\n") & 13);
   Printf (Fmt ("Signed hexadecimal %4X.\n") & 13);
   Printf (Fmt ("Floating point exponential %14.5e.\n") & 13.256555);
   Printf (Fmt ("Floating point decimal %14.5f.\n") & 13.256555);
   Printf (Fmt ("Floating point format %-14.5g.\n") & 13.256555);
   Printf (Fmt ("Single character %-4c.\n") & '@');
   Printf (Fmt ("String %4s.\n") & "Tut");
   Printf (Fmt ("Custom enum type %-4s.\n") & Red);
   Printf (Fmt ("More complexe %% form %02X -> %02.2f.\n") & 13 & 3.145665545255665455);
   Printf (Fmt ("Incorrect format %-4,2s.\n") & Red);
   Printf (Fmt ("Address %s.\n") & Printf'Address);
end EssaiPrintf;
