-------------------------------------------------------------------------------
-- NOM DU CSU (corps)               : formatted_output.adb
-- AUTEUR DU CSU                    : Pascal Pignard
-- VERSION DU CSU                   : 1.0a
-- DATE DE LA DERNIERE MISE A JOUR  : 11 octobre 2014
-- ROLE DU CSU                      : Procédures d'affichage formaté
--
-- FONCTIONS EXPORTEES DU CSU       :
--
--
-- FONCTIONS LOCALES DU CSU         :
--
--
-- NOTES                            : Ada 2012, UTF8, GNATColl, Python
--  Basé sur la fonction de formattage de Python
--
-- COPYRIGHT                        : (c) Pascal Pignard 2014
-- LICENCE                          : CeCILL V2 (http://www.cecill.info)
-- CONTACT                          : http://blady.pagesperso-orange.fr
-------------------------------------------------------------------------------

with System.Address_Image;
package body Formatted_Output is
   use Ada.Strings.Unbounded;

   ---------
   -- Fmt --
   ---------

   function Fmt (Spec : String) return Format is
   begin
      return LFormat : Format do
         LFormat.Format_Spec := Ada.Strings.Unbounded.To_Unbounded_String (Spec);
      end return;
   end Fmt;

   ---------
   -- "&" --
   ---------

   function "&" (Left : Format; Right : Boolean) return Format is
   begin
      return LFormat : Format := Left do
         if LFormat.Arguments = Null_Unbounded_String then
            Append (LFormat.Arguments, Boolean'Image (Right));
         else
            Append (LFormat.Arguments, ',' & Boolean'Image (Right));
         end if;
      end return;
   end "&";

   ---------
   -- "&" --
   ---------

   function "&" (Left : Format; Right : Integer) return Format is
   begin
      return LFormat : Format := Left do
         if LFormat.Arguments = Null_Unbounded_String then
            Append (LFormat.Arguments, Integer'Image (Right));
         else
            Append (LFormat.Arguments, ',' & Integer'Image (Right));
         end if;
      end return;
   end "&";

   ---------
   -- "&" --
   ---------

   function "&" (Left : Format; Right : Float) return Format is
   begin
      return LFormat : Format := Left do
         if LFormat.Arguments = Null_Unbounded_String then
            Append (LFormat.Arguments, Float'Image (Right));
         else
            Append (LFormat.Arguments, ',' & Float'Image (Right));
         end if;
      end return;
   end "&";

   ---------
   -- "&" --
   ---------

   function "&" (Left : Format; Right : Character) return Format is
   begin
      return LFormat : Format := Left do
         if LFormat.Arguments = Null_Unbounded_String then
            Append (LFormat.Arguments, ''' & Right & ''');
         else
            Append (LFormat.Arguments, ',' & ''' & Right & ''');
         end if;
      end return;
   end "&";

   ---------
   -- "&" --
   ---------

   function "&" (Left : Format; Right : String) return Format is
   begin
      return LFormat : Format := Left do
         if LFormat.Arguments = Null_Unbounded_String then
            Append (LFormat.Arguments, ''' & Right & ''');
         else
            Append (LFormat.Arguments, ',' & ''' & Right & ''');
         end if;
      end return;
   end "&";

   ---------
   -- "&" --
   ---------

   function "&" (Left : Format; Right : System.Address) return Format is
   begin
      return LFormat : Format := Left do
         if LFormat.Arguments = Null_Unbounded_String then
            Append (LFormat.Arguments, ''' & System.Address_Image (Right) & ''');
         else
            Append (LFormat.Arguments, ',' & ''' & System.Address_Image (Right) & ''');
         end if;
      end return;
   end "&";

   ----------------
   -- ConcatEnum --
   ----------------

   function ConcatEnum (Left : Format; Right : Element_Type) return Format is
   begin
      return LFormat : Format := Left do
         if LFormat.Arguments = Null_Unbounded_String then
            Append (LFormat.Arguments, ''' & Element_Type'Image (Right) & ''');
         else
            Append (LFormat.Arguments, ',' & ''' & Element_Type'Image (Right) & ''');
         end if;
      end return;
   end ConcatEnum;

   ------------
   -- Printf --
   ------------

   procedure Printf (Fmt : Format) is
      procedure Py_Initialize;
      pragma Import (C, Py_Initialize, "Py_Initialize");
      procedure PyRun_SimpleString (Cmd : String) is
         procedure Internal (Cmd : String);
         pragma Import (C, Internal, "PyRun_SimpleString");
      begin
         Internal (Cmd & ASCII.NUL);
      end PyRun_SimpleString;
      procedure Py_Finalize;
      pragma Import (C, Py_Finalize, "Py_Finalize");
   begin
      Py_Initialize;
      PyRun_SimpleString ("import sys");
      PyRun_SimpleString
        (To_String ("sys.stdout.write('''" & Fmt.Format_Spec & "''' % (" & Fmt.Arguments & "))"));
      Py_Finalize;
   end Printf;

end Formatted_Output;
