-------------------------------------------------------------------------------
-- NOM DU CSU (spécification)       : formatted_output.ads
-- AUTEUR DU CSU                    : Pascal Pignard
-- VERSION DU CSU                   : 1.0a
-- DATE DE LA DERNIERE MISE A JOUR  : 11 octobre 2014
-- ROLE DU CSU                      : Procédures d'affichage formaté
--
-- FONCTIONS EXPORTEES DU CSU       :
--
--
-- FONCTIONS LOCALES DU CSU         :
--
--
-- NOTES                            : Ada 2012, UTF8, GTKAda
--  Basé sur la fonction de formattage de Python
--
-- COPYRIGHT                        : (c) Pascal Pignard 2014
-- LICENCE                          : CeCILL V2 (http://www.cecill.info)
-- CONTACT                          : http://blady.pagesperso-orange.fr
-------------------------------------------------------------------------------

-- http://docs.python.org/2/library/stdtypes.html#string-formatting
--  The conversion flag characters are:
--  Flag	Meaning
--  '#'	The value conversion will use the “alternate form” (where defined below).
--  '0'	The conversion will be zero padded for numeric values.
--  '-'	The converted value is left adjusted (overrides the '0' conversion if both are given).
--  ' '	(a space) A blank should be left before a positive number (or empty string) produced by a signed conversion.
--  '+'	A sign character ('+' or '-') will precede the conversion (overrides a “space” flag).
--  A length modifier (h, l, or L) may be present, but is ignored as it is not necessary for Python –
--    so e.g. %ld is identical to %d.
--
--  The conversion types are:
--
--  Conversion	Meaning	Notes
--  'd'	Signed integer decimal.
--  'i'	Signed integer decimal.
--  'o'	Signed octal value.	(1)
--  'u'	Obsolete type – it is identical to 'd'.	(7)
--  'x'	Signed hexadecimal (lowercase).	(2)
--  'X'	Signed hexadecimal (uppercase).	(2)
--  'e'	Floating point exponential format (lowercase).	(3)
--  'E'	Floating point exponential format (uppercase).	(3)
--  'f'	Floating point decimal format.	(3)
--  'F'	Floating point decimal format.	(3)
--  'g'	Floating point format. Uses lowercase exponential format if exponent
--        is less than -4 or not less than precision, decimal format otherwise.	(4)
--  'G'	Floating point format. Uses uppercase exponential format if exponent
--        is less than -4 or not less than precision, decimal format otherwise.	(4)
--  'c'	Single character (accepts integer or single character string).
--  'r'	String (converts any Python object using repr()).	(5)
--  's'	String (converts any Python object using str()).	(6)
--  '%'	No argument is converted, results in a '%' character in the result.

with System;
private with Ada.Strings.Unbounded;
package Formatted_Output is
   type Format is limited private;
   function Fmt (Spec : String) return Format; -- Without ''' inside
   function "&" (Left : Format; Right : Boolean) return Format;
   function "&" (Left : Format; Right : Integer) return Format;
   function "&" (Left : Format; Right : Float) return Format;
   function "&" (Left : Format; Right : Character) return Format;
   function "&" (Left : Format; Right : String) return Format; -- Without ''' inside
   function "&" (Left : Format; Right : System.Address) return Format; -- With address types use %s

   generic
      type Element_Type is (<>);
   function ConcatEnum (Left : Format; Right : Element_Type) return Format; -- With emumerate types use %s

   procedure Printf (Fmt : Format);
private
   type Format is record
      Format_Spec : Ada.Strings.Unbounded.Unbounded_String;
      Arguments   : Ada.Strings.Unbounded.Unbounded_String;
   end record;
end Formatted_Output;
